import socket
import uuid
from datetime import datetime

import requests
import xbmcplugin
from xbmcgui import Dialog

from resources.lib.api.gitlab_api import GitLabAPI
from resources.lib.const import SETTINGS, LANG, GENERAL, ROUTE
from resources.lib.defaults import Defaults
from resources.lib.gui.renderers.dialog_renderer import DialogRenderer
from resources.lib.kodilogging import logger, setup_root_logger
from resources.lib.routing.router import router
from resources.lib.storage.settings import settings
from resources.lib.stream_cinema import StreamCinema
from resources.lib.utils.kodiutils import get_string, set_settings, get_current_datetime_str, \
    datetime_from_iso, get_info, time_limit_expired, clear_kodi_addon_cache, get_kodi_language

provider = Defaults.provider()
api = Defaults.api()
gitlab_api = GitLabAPI()
stream_cinema = StreamCinema(provider, api)


def run(argv):
    setup_root_logger()
    url, handle, query = parse_argv(argv)
    try:
        first_run()
        run_checks()
        stream_cinema.dispatch(handle, url, query)
    except requests.exceptions.ConnectionError as e:
        logger.error(e)
        if can_connect_google():
            Dialog().ok(get_string(LANG.CONNECTION_ERROR), get_string(LANG.SERVER_ERROR_HELP))
        else:
            Dialog().ok(get_string(LANG.CONNECTION_ERROR), get_string(LANG.NO_CONNECTION_HELP))
        # TODO: Redirect to previous page (needs some smart way to remember previous page)
        xbmcplugin.endOfDirectory(handle, cacheToDisc=False)
        router.go(ROUTE.ROOT)


def parse_argv(argv):
    return argv[0], int(argv[1]), argv[2]


def run_checks():
    stream_cinema.vip_remains()
    check_version()


def first_run():
    set_uuid()
    set_installation_date()
    set_version()
    set_default_language()


def set_default_language():
    if settings[SETTINGS.PREFERRED_LANGUAGE] == '0':
        settings.set_select(SETTINGS.PREFERRED_LANGUAGE, get_kodi_language())


def set_version():
    set_settings(SETTINGS.VERSION, get_info('version'))


def set_installation_date():
    if settings[SETTINGS.INSTALLATION_DATE] == '':
        set_settings(SETTINGS.INSTALLATION_DATE, get_current_datetime_str())
        stream_cinema.intro()


def set_uuid():
    if settings[SETTINGS.UUID] == '':
        settings[SETTINGS.UUID] = uuid.uuid4()


def check_version():
    expired = time_limit_expired(SETTINGS.LAST_VERSION_CHECK, GENERAL.VERSION_CHECK_INTERVAL)
    if expired or settings[SETTINGS.LAST_VERSION_AVAILABLE] == '':
        settings[SETTINGS.LAST_VERSION_CHECK] = datetime.now()
        tag_name = get_latest_release_tag_name()
        if tag_name:
            settings[SETTINGS.LAST_VERSION_AVAILABLE] = tag_name
            current_version = settings[SETTINGS.VERSION]
            if current_version != tag_name:
                settings[SETTINGS.IS_OUTDATED] = True
                clear_kodi_addon_cache()
                DialogRenderer.ok(get_string(LANG.NEW_VERSION_TITLE),
                                  get_string(LANG.NEW_VERSION_TEXT).format(version=tag_name))
            else:
                settings[SETTINGS.IS_OUTDATED] = False


def get_latest_release_tag_name():
    releases = gitlab_api.get_latest_release()
    latest_release = max(releases, key=lambda x: datetime_from_iso(x['released_at']))
    if latest_release:
        return latest_release.get('tag_name')


def can_connect_google():
    google_addr = ("www.google.com", 443)
    soc = None
    try:
        soc = socket.create_connection(google_addr)
        return True
    except Exception as e:
        logger.debug("failed to connect to %s: %s", google_addr, e)
        return False
    finally:
        if soc is not None:
            soc.close()
